@@ -75,35 +75,6 @@ typedef struct {
  * BINARY ENCRYPTED FILE FORMAT
  */
 
-static void
-init_salt (guchar salt[8])
-{
-	gboolean got_random;
-	int i, fd;
-
-	got_random = FALSE;
-#ifdef HAVE_DEVRANDOM
-	fd = open ("/dev/random", O_RDONLY);
-	if (fd != -1) {
-		struct stat st;
-		/* Make sure it's a character device */
-		if ((fstat (fd, &st) == 0) && S_ISCHR (st.st_mode)) {
-			if (read (fd, salt, 8) == 8) {
-				got_random = TRUE;
-			}
-		}
-		close (fd);
-	}
-#endif
-
-	if (!got_random) {
-		for (i=0; i < 8; i++) {
-			salt[i] = (int) (256.0*rand()/(RAND_MAX+1.0));
-		}
-	}
-	
-}
-
 static gboolean
 encrypt_buffer (EggBuffer *buffer,
 		const char *password,
@@ -302,7 +273,7 @@ gkr_keyring_binary_generate (GkrKeyring *keyring, EggBuffer *buffer)
 	/* Prepare the keyring for encryption */
 	if (!keyring->salt_valid) {
 		keyring->hash_iterations = 1000 + (int) (1000.0 * rand() / (RAND_MAX + 1.0));
-		init_salt (keyring->salt);
+		gcry_create_nonce (keyring->salt, sizeof (keyring->salt));
 		keyring->salt_valid = TRUE;
 	}	
 		
@@ -409,17 +380,16 @@ decode_acl (EggBuffer *buffer, gsize offset, gsize *offset_out, GList **out)
 			g_free (name);
 			goto bail;
 		}
-		if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &reserved) ||
-		    reserved != NULL) {
+		reserved = NULL;
+		if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &reserved)) {
 			g_free (name);
 			g_free (path);
-			g_free (reserved);
 			goto bail;
 		}
+		g_free (reserved);
 		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &y)) {
 			g_free (name);
 			g_free (path);
-			g_free (reserved);
 			goto bail;
 		}
 
@@ -464,6 +434,7 @@ gkr_keyring_binary_parse (GkrKeyring *keyring, EggBuffer *buffer)
 	guint32 lock_timeout;
 	time_t mtime, ctime;
 	char *display_name;
+	gsize n_secret;
 	int i, j;
 	guint32 tmp;
 	guint32 num_items;
@@ -525,13 +496,8 @@ gkr_keyring_binary_parse (GkrKeyring *keyring, EggBuffer *buffer)
 	}
 	
 	for (i = 0; i < 4; i++) {
-		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &tmp)) {
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &tmp))
 			goto bail;
-		}
-		/* reserved bytes must be zero */
-		if (tmp != 0) {
-			goto bail;
-		}
 	}
 	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &num_items)) {
 		goto bail;
@@ -558,11 +524,9 @@ gkr_keyring_binary_parse (GkrKeyring *keyring, EggBuffer *buffer)
 					     &crypto_size)) {
 		goto bail;
 	}
-	/* Make sure the rest of the file is the crypted part only */
-	if (crypto_size % 16 != 0 ||
-	    buffer->len - offset != crypto_size) {
+	/* Make the crypted part is the right size */
+	if (crypto_size % 16 != 0)
 		goto bail;
-	}
 	
 	/* Copy the data into to_decrypt into non-pageable memory */
 	egg_buffer_init_static (&to_decrypt, buffer->buf + offset, crypto_size);
@@ -584,10 +548,18 @@ gkr_keyring_binary_parse (GkrKeyring *keyring, EggBuffer *buffer)
 				                                &items[i].display_name)) {
 					goto bail;
 				}
-				if (!gkr_proto_get_utf8_secret (buffer, offset, &offset,
-				                                &items[i].secret)) {
+				if (!gkr_proto_get_raw_secret (buffer, offset, &offset,
+				                               (guchar**)(&items[i].secret), &n_secret)) {
 					goto bail;
 				}
+				/* We don't support binary secrets yet, skip */
+				if (!g_utf8_validate ((gchar*)items[i].secret, n_secret, NULL)) {
+					g_message ("discarding item with unsupported non-textual secret: %s", 
+					           items[i].display_name);
+					free (items[i].display_name);
+					free (items[i].secret);
+					continue;
+				}
 				if (!gkr_proto_get_time (buffer, offset, &offset,
 				                         &items[i].ctime)) {
 					goto bail;
@@ -597,21 +569,13 @@ gkr_keyring_binary_parse (GkrKeyring *keyring, EggBuffer *buffer)
 					goto bail;
 				}
 				reserved = NULL;
-				if (!gkr_proto_get_utf8_string (buffer, offset, &offset,
-				                                &reserved) ||
-				    reserved != NULL) {
-					g_free (reserved);
+				if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &reserved))
 					goto bail;
-				}
+				g_free (reserved);
 				for (j = 0; j < 4; j++) {
 					guint32 tmp;
-					if (!egg_buffer_get_uint32 (buffer, offset, &offset, &tmp)) {
+					if (!egg_buffer_get_uint32 (buffer, offset, &offset, &tmp))
 						goto bail;
-					}
-					/* reserved bytes must be zero */
-					if (tmp != 0) {
-						goto bail;
-					}
 				}
 				if (!gkr_proto_decode_attribute_list (buffer, offset, &offset,
 				                                      &items[i].attributes)) {
