@@ -26,8 +26,8 @@
 
 gboolean      gkr_pkcs11_daemon_initialize       (void);
 
-gboolean      gkr_pkcs11_daemon_setup_pkcs11     (void);
+gboolean      gkr_pkcs11_daemon_startup_pkcs11   (void);
 
-gboolean      gkr_pkcs11_daemon_setup_ssh        (void);
+gboolean      gkr_pkcs11_daemon_startup_ssh      (void);
 
 #endif /* GKRPKCS11DAEMON_H_ */
