@@ -29,6 +29,8 @@
 
 #include "gnome-keyring-result.h"
 
+G_BEGIN_DECLS
+
 #define GNOME_KEYRING_SESSION   "session"
 #define GNOME_KEYRING_DEFAULT   NULL
 
@@ -516,4 +518,6 @@ GnomeKeyringResult    gnome_keyring_daemon_set_display_sync         (const char
 
 GnomeKeyringResult    gnome_keyring_daemon_prepare_environment_sync (void);
 
+G_END_DECLS
+
 #endif /* GNOME_KEYRING_H */
