@@ -24,13 +24,14 @@
 #include "pkcs11/pkcs11.h"
 
 #include "gck-attributes.h"
+#include "gck-authenticator.h"
 #include "gck-factory.h"
 #include "gck-private-key.h"
+#include "gck-session.h"
 #include "gck-transaction.h"
 #include "gck-util.h"
 
 struct _GckPrivateKeyPrivate {
-	guint sexp_uses;
 	GckSexp *sexp;
 };
 
@@ -142,14 +143,62 @@ done:
 	return ret;
 }
 
+static void
+factory_create_private_key (GckSession *session, GckTransaction *transaction, 
+                            CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs, GckObject **object)
+{
+	GckPrivateKey *key;
+	GckSexp *sexp;
+	
+	g_return_if_fail (GCK_IS_TRANSACTION (transaction));
+	g_return_if_fail (attrs || !n_attrs);
+	g_return_if_fail (object);
+
+	sexp = gck_private_key_create_sexp (session, transaction, attrs, n_attrs);
+	if (sexp == NULL)
+		return;
+	
+	key = g_object_new (GCK_TYPE_PRIVATE_KEY, "base-sexp", sexp,
+	                    "module", gck_session_get_module (session), NULL);
+	key->pv->sexp = sexp;
+	*object = GCK_OBJECT (key);
+}
+
+static gboolean
+acquire_from_authenticator (GckAuthenticator *auth, GckObject *object, gpointer user_data)
+{
+	GckSexp **result = user_data;
+
+	g_assert (result);
+	g_assert (!*result);
+
+	/* The sexp we stored on the authenticator */
+	*result = g_object_get_data (G_OBJECT (auth), "private-key-sexp");
+	if (*result != NULL) {
+		*result = gck_sexp_ref (*result);
+		gck_authenticator_throw_away_one_use (auth);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+have_from_authenticator (GckAuthenticator *auth, GckObject *object, gpointer unused)
+{
+	/* The sexp we stored on the authenticator */
+	return g_object_get_data (G_OBJECT (auth), "private-key-sexp") ? TRUE : FALSE;
+}
+
 /* -----------------------------------------------------------------------------
  * PRIVATE_KEY 
  */
 
 static CK_RV 
-gck_private_key_real_get_attribute (GckObject *base, CK_ATTRIBUTE* attr)
+gck_private_key_real_get_attribute (GckObject *base, GckSession *session, CK_ATTRIBUTE* attr)
 {
 	GckPrivateKey *self = GCK_PRIVATE_KEY (base);
+	gboolean have;
 	
 	switch (attr->type) {
 	case CKA_CLASS:
@@ -189,7 +238,10 @@ gck_private_key_real_get_attribute (GckObject *base, CK_ATTRIBUTE* attr)
 		return CKR_ATTRIBUTE_TYPE_INVALID;
 		
 	case CKA_ALWAYS_AUTHENTICATE:
-		return gck_attribute_set_bool (attr, self->pv->sexp_uses <= 1);
+		have = self->pv->sexp ? TRUE : FALSE;
+		if (!have && session)
+			have = gck_session_for_each_authenticator (session, base, have_from_authenticator, NULL);
+		return gck_attribute_set_bool (attr, !have);
 		
 	case CKA_MODULUS:
 		return gck_key_set_key_part (GCK_KEY (self), GCRY_PK_RSA, "n", attr);
@@ -220,30 +272,24 @@ gck_private_key_real_get_attribute (GckObject *base, CK_ATTRIBUTE* attr)
 		return CKR_ATTRIBUTE_SENSITIVE;
 	};	
 	
-	return GCK_OBJECT_CLASS (gck_private_key_parent_class)->get_attribute (base, attr);
+	return GCK_OBJECT_CLASS (gck_private_key_parent_class)->get_attribute (base, session, attr);
 }
 
 static GckSexp*
-gck_private_key_real_acquire_crypto_sexp (GckKey *base)
+gck_private_key_real_acquire_crypto_sexp (GckKey *base, GckSession *session)
 {
 	GckPrivateKey *self = GCK_PRIVATE_KEY (base);
-	GckSexp *sexp;
+	GckSexp *sexp = NULL;
 	
-	if (self->pv->sexp_uses == 0) {
-		g_return_val_if_fail (!self->pv->sexp, NULL);
-		return NULL;
-	}
+	/* We have an unlocked private key here */
+	if (self->pv->sexp)
+		sexp = gck_sexp_ref (self->pv->sexp);
+
+	/* Find an authenticator, with an unlocked copy */
+	else
+		gck_session_for_each_authenticator (session, GCK_OBJECT (self),
+		                                    acquire_from_authenticator, &sexp);
 	
-	g_return_val_if_fail (self->pv->sexp, NULL);
-		
-	sexp = gck_sexp_ref (self->pv->sexp);
-	--(self->pv->sexp_uses);
-		
-	if (self->pv->sexp_uses == 0) {
-		gck_sexp_unref (self->pv->sexp);
-		self->pv->sexp = NULL;
-	}
-		
 	return sexp;
 }
 
@@ -335,38 +381,46 @@ gck_private_key_class_init (GckPrivateKeyClass *klass)
  */
 
 void
-gck_private_key_store_private (GckPrivateKey *self, GckSexp *sexp, guint num_uses)
+gck_private_key_set_unlocked_private (GckPrivateKey *self, GckSexp *sexp)
 {
 	g_return_if_fail (GCK_IS_PRIVATE_KEY (self));
-	g_return_if_fail (!sexp || num_uses);
-	
+	g_return_if_fail (sexp);
+
 	if (sexp)
 		gck_sexp_ref (sexp);
-	if (self->pv->sexp) 
+	if (self->pv->sexp)
 		gck_sexp_unref (self->pv->sexp);
 	self->pv->sexp = sexp;
-	self->pv->sexp_uses = num_uses;
 }
 
-
 void
-gck_private_key_create (GckSession *session, GckTransaction *transaction, 
-                        CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs, GckObject **object)
+gck_private_key_set_locked_private (GckPrivateKey *self, GckAuthenticator *auth, 
+                                    GckSexp *sexp)
+{
+	g_return_if_fail (GCK_IS_PRIVATE_KEY (self));
+	g_return_if_fail (GCK_IS_AUTHENTICATOR (auth));
+
+	if (sexp == NULL)
+		g_object_set_data (G_OBJECT (auth), "private-key-sexp", NULL);
+	else
+		g_object_set_data_full (G_OBJECT (auth), "private-key-sexp",
+		                        gck_sexp_ref (sexp), gck_sexp_unref);
+}
+
+GckSexp*
+gck_private_key_create_sexp (GckSession *session, GckTransaction *transaction, 
+                             CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs)
 {
  	CK_KEY_TYPE type;
- 	GckSexp *wrapper;
  	gcry_sexp_t sexp;
  	CK_RV ret;
  	
-	g_return_if_fail (GCK_IS_TRANSACTION (transaction));
-	g_return_if_fail (attrs || !n_attrs);
-	g_return_if_fail (object);
-	
-	*object = NULL;
+	g_return_val_if_fail (GCK_IS_TRANSACTION (transaction), NULL);
+	g_return_val_if_fail (attrs || !n_attrs, NULL);
 	
 	if (!gck_attributes_find_ulong (attrs, n_attrs, CKA_KEY_TYPE, &type)) {
 		gck_transaction_fail (transaction, CKR_TEMPLATE_INCOMPLETE);
-		return;
+		return NULL;
 	}
 		
  	gck_attributes_consume (attrs, n_attrs, CKA_KEY_TYPE, CKA_CLASS, G_MAXULONG);
@@ -386,31 +440,26 @@ gck_private_key_create (GckSession *session, GckTransaction *transaction,
  	
 	if (ret != CKR_OK) {
 		gck_transaction_fail (transaction, ret);
-		return;
+		return NULL;
 	}
 	
-	g_return_if_fail (sexp);
-	wrapper = gck_sexp_new (sexp);
-	*object = g_object_new (GCK_TYPE_PRIVATE_KEY, "base-sexp", wrapper, NULL);
-	gck_private_key_store_private (GCK_PRIVATE_KEY (*object), wrapper, G_MAXUINT);
-	gck_sexp_unref (wrapper);
+	g_return_val_if_fail (sexp, NULL);
+	return gck_sexp_new (sexp);
 }
 
 GckFactoryInfo*
 gck_private_key_get_factory (void)
 {
 	static CK_OBJECT_CLASS klass = CKO_PRIVATE_KEY;
-	static CK_BBOOL token = CK_FALSE;
 
 	static CK_ATTRIBUTE attributes[] = {
-		{ CKA_CLASS, &klass, sizeof (klass) },
-		{ CKA_TOKEN, &token, sizeof (token) }, 
+		{ CKA_CLASS, &klass, sizeof (klass) }
 	};
 
 	static GckFactoryInfo factory = {
 		attributes,
 		G_N_ELEMENTS (attributes),
-		gck_private_key_create
+		factory_create_private_key
 	};
 	
 	return &factory;
