@@ -52,17 +52,20 @@ struct _GckSessionClass {
 #endif
 };
 
+typedef gboolean         (*GckAuthenticatorFunc)                        (GckAuthenticator *auth,
+                                                                         GckObject *object,
+                                                                         gpointer user_data);
+
 GType                    gck_session_get_type                           (void);
 
-GckSession*              gck_session_new                                (GckModule *module, 
-                                                                         GckManager *manager,
-                                                                         CK_SLOT_ID slot_id,
-                                                                         CK_FLAGS flags);
+GckSession*              gck_session_for_session_object                 (GckObject *obj);
 
 CK_SESSION_HANDLE        gck_session_get_handle                         (GckSession *self);
 
 CK_SLOT_ID               gck_session_get_slot_id                        (GckSession *self);
 
+CK_ULONG                 gck_session_get_apartment                      (GckSession *self);
+
 GckModule*               gck_session_get_module                         (GckSession *self);
 
 GckManager*              gck_session_get_manager                        (GckSession *self);
@@ -86,7 +89,14 @@ CK_RV                    gck_session_login_context_specific             (GckSess
                                                                          CK_UTF8CHAR_PTR pin,
                                                                          CK_ULONG n_pin);
 
+void                     gck_session_destroy_session_object             (GckSession *self,
+                                                                         GckTransaction *transaction,
+                                                                         GckObject *obj);
 
+gboolean                 gck_session_for_each_authenticator             (GckSession *self,
+                                                                         GckObject *object,
+                                                                         GckAuthenticatorFunc func,
+                                                                         gpointer user_data);
 
 
 
