@@ -208,7 +208,7 @@ static pthread_mutex_t call_state_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /* Allocator for call session buffers */
 static void*
-call_allocator (void* p, unsigned long sz)
+call_allocator (void* p, size_t sz)
 {
 	void* res = realloc (p, (size_t)sz);
 	if (!res && sz)
@@ -225,7 +225,6 @@ call_connect (CallState *cs)
 	assert (cs);
 	assert (cs->socket == -1);
 	assert (cs->call_status == CALL_INVALID);
-	assert (pkcs11_socket_path[0]);
 	
 	debug (("connecting to: %s", pkcs11_socket_path));
 		
@@ -645,7 +644,7 @@ proto_read_attribute_array (GckRpcMessage *msg, CK_ATTRIBUTE_PTR arr, CK_ULONG l
 	assert (msg);
 
 	/* Make sure this is in the right order */
-	assert (!msg->signature || gck_rpc_message_verify_part (msg, "aAu"));
+	assert (!msg->signature || gck_rpc_message_verify_part (msg, "aA"));
 	
 	/* Get the number of items. We need this value to be correct */
 	if (!egg_buffer_get_uint32 (&msg->buffer, msg->parsed, &msg->parsed, &num))
@@ -732,10 +731,10 @@ proto_read_attribute_array (GckRpcMessage *msg, CK_ATTRIBUTE_PTR arr, CK_ULONG l
 		return PARSE_ERROR;
 	
 	/* Read in the code that goes along with these attributes */
-	if (!egg_buffer_get_uint32 (&msg->buffer, msg->parsed, &msg->parsed, &num))
+	if (!gck_rpc_message_read_ulong (msg, &ret))
 		return PARSE_ERROR;
 
-	return (CK_RV)num;
+	return ret;
 }
 
 static CK_RV
@@ -758,7 +757,7 @@ proto_read_byte_array (GckRpcMessage *msg, CK_BYTE_PTR arr,
 	
 	/* If not valid, then just the length is encoded, this can signify CKR_BUFFER_TOO_SMALL */
 	if (!valid) {
-		if (!egg_buffer_get_uint32 (&msg->buffer, msg->parsed, &msg->parsed, &vlen))
+		if (!egg_buffer_get_uint32 (&msg->buffer, msg->parsed, &msg->parsed, (uint32_t *)&vlen))
 			return PARSE_ERROR;
 		
 		*len = vlen;
@@ -791,7 +790,8 @@ static CK_RV
 proto_read_ulong_array (GckRpcMessage *msg, CK_ULONG_PTR arr,
                         CK_ULONG_PTR len, CK_ULONG max)
 {
-	uint32_t i, num, val;
+	uint32_t i, num;
+	uint64_t val;
 	unsigned char valid;
 
 	assert (len);
@@ -823,9 +823,9 @@ proto_read_ulong_array (GckRpcMessage *msg, CK_ULONG_PTR arr,
 
 	/* We need to go ahead and read everything in all cases */
 	for (i = 0; i < num; ++i) {
-		egg_buffer_get_uint32 (&msg->buffer, msg->parsed, &msg->parsed, &val);
+		egg_buffer_get_uint64 (&msg->buffer, msg->parsed, &msg->parsed, &val);
 		if (arr)
-			arr[i] = val;
+			arr[i] = (CK_ULONG)val;
 	}
 
 	return egg_buffer_has_error (&msg->buffer) ? PARSE_ERROR : CKR_OK;
