@@ -39,6 +39,7 @@ void test_mainloop_quit (void);
 void test_mainloop_run (int timeout);
 GMainLoop* test_mainloop_get (void);
 
+const gchar* test_dir_testdata (void);
 guchar* test_read_testdata (const gchar *basename, gsize *n_data);
 gchar* test_build_filename (const gchar *basename);
 
@@ -57,6 +58,16 @@ gchar* test_build_filename (const gchar *basename);
 #define DEFINE_TEST(x) \
 	void test_##x(int *__unused G_GNUC_UNUSED, gconstpointer __data G_GNUC_UNUSED)
 
+#define DECLARE_START(x) \
+	void start_##x(void)
+#define DEFINE_START(x) \
+	void start_##x(void)
+
+#define DECLARE_STOP(x) \
+	void stop_##x(void)
+#define DEFINE_STOP(x) \
+	void stop_##x(void)
+
 /* #define DEFINE_ABORT(x) void abort_x(void *__unused G_GNUC_UNUSED, gconstpointer __data G_GNUC_UNUSED)' */
 
 #endif /* GTEST_HELPERS_H_ */
